;**********************************************************************
;   								      *
;    Test emision/detection IR                                        *
;	A 1.2 ms						      *
;      	0.6ms a 1, 0.6 a 0 pour un 1				      *
;	1.2 ms a 0 pour un 0					      *
;                                                                     *
;	1er test detection IR                                         *
;							              *
;**********************************************************************
;                                                                     *
;    Filename:	    IrDetec.asm                                       *
;    Date:          25/01/00                                          *
;    File Version:  1                                                 *
;                                                                     *
;    Author:        Julien                                            *
;    Company:       Fribotte                                          *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: rien                                             *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: Quartz 4 Mhz                                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
; Pas de code protege, pas de watch hdog timer, delai de demarrage, horloge XT.


;**********************************************************************
;                                                                     *
;    Variables                                                        *
;                                                                     *
;                                                                     *
;**********************************************************************


;***** VARIABLE IR
CompteurIR	EQU 	0x13		; Compteur IR pour mesurer 256 emissions
ValeurIr	EQU 	0x17		; Valeur IR a fournir pendant 1 cycle
ResultatIr	EQU 	0x19		; Passe a 0 par la boucle d'emission IR si resultat pas bon.
BrouillageIR	EQU 	0x1A		; Passe a 1 par la boucle d'emission IR si brouillage detecte.
Nb1DetectIr	EQU 	0x1B		; Nombre de detection de "1" effectuee dans la boucle d'emission IR (1/2 periode)


;**************************************************************** DEBUT
		ORG     0x000           ; processor reset vector
  		GOTO    main            ; go to beginning of program



;**********************************************************************
;                                                                     *
;    Interruption Timer                                               *
;                                                                     *
;                                                                     *
;**********************************************************************

		ORG     0x004           ; interrupt vector location
					; Ici on s'en sert pas !
		RETFIE                  ; return from interrupt



;**********************************************************************
;                                                                     *
;    Programme Main                                                   *
;                                                                     *
;                                                                     *
;**********************************************************************


main

;***** initialisations
	CALL	InitIO				; On intialise les ports d'E/S (A et B)
	
boucle

TestCapteurIr   CALL Emission8Bits	; On emet les 8 bits ...
		
		; On recopie le resultat des tests sur le port A.

		BCF	PORTB, 1	; On met a 0 (tres peu de temps) les BIT resustats	
		BCF	PORTB, 2	;	/

		MOVF 	ResultatIr,0	; detection d'un obstacle ?
		BTFSS	STATUS,2	; 
		BSF	PORTB, 1	; Cas ResultatIr = 1 , on passe le bit 1 du PORT B a 1 (Detection)
		MOVF 	BrouillageIR,0	; Signal brouille ?
		BTFSS	STATUS,2	; 
		BSF	PORTB, 2	; Cas BrouillageIR = 1 , on passe le bit 2 du PORT B a 1 (Brouillage)

		GOTO 	boucle				; Boucle infinie du main

;**********************************************************************
;                                                                     *
;    Emission 8 Bits		 	                       	      *
;                                                                     *
;                                                                     *
;**********************************************************************


Emission8Bits	

		MOVLW	0x1		; ResultatIr a 1. Il sera mis a 0 si il n'y a pas de retour du signal.
		MOVWF	ResultatIr	;	/				
		MOVLW	0x0		; BrouillageIR a 0. Il sera mis a 1 si le signal est brouillee.
		MOVWF	BrouillageIR	;	/				
		MOVLW	0xFF		; ValeurIr a 255
		MOVWF	ValeurIr	;	/
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1"
		CALL	EmissionIr	; On emet le "1" - 8 bits en tout !

	RETURN

;**********************************************************************
;                                                                     *
;    Emission IR 	Periode 	                       	      *
;                                                                     *
;                                                                     *
;**********************************************************************

;***** Emission d'un "bit" IR en 1.2 ms
; Valeur d'emission indique dans ValeurIr

EmissionIr
		CALL 	EmissionIrDP	; Emet ValeurIr pensant 0.6 ms
		CALL 	TestResultIR	; Test du resultat 			
		MOVLW	0x0		; ValeurIr a 0
		MOVWF	ValeurIr	;	/
		CALL 	EmissionIrDP	; Emet 0 pensant 0.6 ms
		CALL 	TestResultIR	; Test du resultat 
	RETURN

;**********************************************************************
;                                                                     *
;    Emission IR 	Test du rsultat                       	      *
;                                                                     *
;                                                                     *
;**********************************************************************

TestResultIR	
		MOVF 	ValeurIr,0	; On teste ce qui etait demande (ValeurIr)
		BTFSC	STATUS,2	; 
		GOTO	TestEmissionA0 	; Cas EmissionIrDP = 0  
					
TestEmissionA1				; Cas EmissionIrDP = 1
		MOVF 	Nb1DetectIr,0	; On teste ce qui a ete recu
		SUBLW	0x11		; On lui enleve 17
		BTFSS	STATUS,0	; Carry = 0 ?	
		GOTO	TestEmissIrOK	; carry = 0, cad 18 - Nb1DetectIr <0 , cad Nb1DetectIr > 17
		GOTO	TestEmissIrKO	; carry = 1, cad 18 - Nb1DetectIr >=0 , cad Nb1DetectIr <= 17

TestEmissionA0				; Cas EmissionIrDP = 0
		MOVF 	Nb1DetectIr,0	; On teste ce qui a ete recu
		SUBLW	0x5		; On lui enleve 5
		BTFSS	STATUS,0	; Carry = 0 ?	
		GOTO	TestEmissIrBr	; carry = 0, cad 18 - Nb1DetectIr <0 , cad Nb1DetectIr > 5
		GOTO	TestEmissIrOK	; carry = 1, cad 18 - Nb1DetectIr >=0 , cad Nb1DetectIr <= 5

TestEmissIrBr	MOVLW	0x1		; emission Brouille !
		MOVWF	BrouillageIR	;	/	

TestEmissIrKO	MOVLW	0x0		; Pas d'obstacle detecte !
		MOVWF	ResultatIr	;	/	

TestEmissIrOK				; Fini

	RETURN


;**********************************************************************
;                                                                     *
;    Emission IR 1/2 Periode 	                          	      *
;                                                                     *
;                                                                     *
;**********************************************************************

;***** Emission IR a 40 Khz ; Doit durer 2*13*s (le 12.5 etant impossible).
; 23 passes ( 23 * 26 s = 1.2 ms)
; La valeur de l'emission (0 ou 1) est passe en parametre dans ValeurIr

EmissionIrDP
		MOVLW	0x17				; Compteur a 23
		MOVWF	CompteurIR			;	/
		MOVLW	0x0				; Nb1DetectIR a 0
		MOVWF	Nb1DetectIr			;	/

BoucleIR	
		BTFSC	ValeurIr,0	; 1 s On teste le bit 0 de Valeur IR
		BSF	PORTB, 4	; 2 s Si valeurIe = 1, on met le bit 4 du PORT B a 1, sinon on passe (ici un NOP est execute, donc 2s quand meme)
		NOP			; 3 s
		NOP			; 4 s
		NOP			; 5 s
		NOP			; 6 s
		NOP			; 7 s
		NOP			; 8 s
		NOP			; 9 s
		NOP			; 10 s
		NOP			; 11 s
		NOP			; 12 s
		NOP			; 13 s

		BCF	PORTB, 4	; 1 s on met le bit 4 du PORT B a 0
		NOP			; 2 s
		NOP			; 3 s
		NOP			; 4 s
		NOP			; 5 s
		NOP			; 6 s
		BTFSS	PORTA, 0	; On teste l'etat du bit 0 du port A (capteur) 7 ms
		INCF 	Nb1DetectIr	; Il est a 0 (cad detection car le capteur est inversee),
					;	on incremente Nb1DetectIR 8s
		NOP			; On continue 9s
		DECF	CompteurIR	; 10 s
		BTFSS	STATUS,2	; CompteurIR = 0 ? 11 s
		GOTO	BoucleIR	; Bit =0, cad resultat non nul, on continu  13 s

	RETURN


;**********************************************************************
;                                                                     *
;    Initialisation des ports d'E/S                           	      *
;                                                                     *
;                                                                     *
;**********************************************************************


InitIO
	CLRF	PORTB		; Initialise le port B
	BSF	STATUS, RP0	; Bank 1 selectonnee
	MOVLW	0x0		; Port RB7-RB0 en sortie
	MOVWF	TRISB		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee

	CLRF 	PORTA		; Initialise le port A
	BSF 	STATUS,	RP0	; Bank 1 selectonnee
	MOVLW	0x01		; Port RA0 en entree RA1-RA4 en sortie
	MOVWF	TRISA		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee
	RETURN


;**********************************************************************
;                                                                     *
;    Fin				                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	END                     ; directive 'end of program'


